import logging
import smtplib
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from pathlib import Path
from typing import Optional

from pydantic import BaseModel, Field

logger = logging.getLogger(__name__)


class MailSender(BaseModel):
    """Classe représentant un objet permettant d'envoyer un e-mail via SMTP."""

    smtp_server: str
    sender_email: str
    smtp_port: int = Field(default=25)
    login_auth: str = Field(default="")
    password_auth: str = Field(default="", repr=False)

    def send(
        self,
        dest_mail: str,
        subject: str,
        cc: str = "",
        bcc: str = "",
        message: str = "",
        type_message: str = "plain",
        file_attachement: Optional[list[str]] = None,
    ) -> None:
        """Envoie un e-mail au destinataire spécifié.

        Args:
            dest_mail (str): L'adresse e-mail du destinataire
            subject (str): Le sujet de l'e-mail
            cc (str) : adresse copie
            bcc (str) : adresse copie invisible
            message (str): Le contenu du message de l'e-mail
            type_message (str): type du message à envoyer ('plain' ou 'html')
            file_attachement (list[str]): Liste des fichiers à attacher au mail

        Raises:
            Exception: Si une erreur se produit lors de l'envoi de l'e-mail.
        """
        msg = MIMEMultipart()
        msg["From"] = self.sender_email
        msg["To"] = dest_mail
        msg["Cc"] = cc
        msg["Bcc"] = bcc
        msg["Subject"] = subject

        msg.attach(MIMEText(message, type_message, "utf-8"))

        if file_attachement:
            for element in file_attachement:
                if Path(element).exists():
                    with Path(element).open("rb") as file:
                        attachment = MIMEApplication(file.read())
                    attachment.add_header("Content-Disposition", "attachment", filename=Path(element).name)
                    msg.attach(attachment)

        try:
            with smtplib.SMTP(self.smtp_server, self.smtp_port) as server:
                if self.login_auth and self.password_auth:
                    server.login(self.login_auth, self.password_auth)
                server.send_message(msg)
                logger.info("E-mail envoyé avec succès")
        except smtplib.SMTPResponseException:
            logger.debug("Erreur lors de l'envoi de l'e-mail")
            logger.debug(
                "Erreur [%s]: %s", smtplib.SMTPResponseException.smtp_code, smtplib.SMTPResponseException.smtp_error
            )
